import java.awt.*;
import java.awt.event.*;
import java.io.*;
import ij.*;
import ij.io.*;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;

public class OpenMPA_ implements PlugIn {

	public void run(String arg) {
		OpenDialog od = new OpenDialog("Open MPA...", arg);
		String file = od.getFileName();
		if (file == null) return;
		String directory = od.getDirectory();
		ImagePlus imp = open(directory, file);
		if (imp != null ) {
			imp.show();
		} else {
			IJ.showMessage("Open MPA...", "Failed.");
		}
	}

	public static ImagePlus open(String directory, String file) {
		File f = new File(directory, file);
		try {
			FileInputStream in = new FileInputStream(f);
			FileInfo fi = new FileInfo();
			fi.directory = directory;
			fi.fileFormat = fi.RAW;
			fi.fileName = file;
			fi.fileType = FileInfo.GRAY32_INT;
			fi.gapBetweenImages = 0;
			fi.height = 512;
			fi.width = 512;
			fi.intelByteOrder = true;
			fi.nImages = 1;

			String line;
			int i=0;
			int pos1=0;
			int flagADC=0;
			int value;
			String substr;
			BufferedReader br = new BufferedReader(new FileReader(f));
			line = br.readLine();
			while (line!= null)
			{
				if (line.indexOf("[ADC1]")!=-1)
				{
					flagADC=1;
					//IJ.log("ADC1");
				}
				if (line.indexOf("[ADC2]")!=-1)
				{
					flagADC=2;
					//IJ.log("ADC2");
				}
				if (line.indexOf("range=")!=-1)
				{
					substr=line.substring(line.lastIndexOf('=')+1, line.length());
					try{
						value=Integer.parseInt(substr);
					}
					catch (NumberFormatException e) {
						IJ.log("error.. "+substr);
						value=512;
					}
					//IJ.log("range : "+value);
					if (flagADC==1)
					{
						fi.width = value;
					}
					if (flagADC==2)
					{
						fi.height = value;
						break;
						//
					}
				}
				line = br.readLine();
				//IJ.log(line);
			}
			br.close();
			long header=f.length()-(fi.width*fi.height*4); // detection taille image
			fi.offset = (int)(header);
			FileOpener fo = new FileOpener(fi);
			ImagePlus imp = fo.open(false);
			IJ.showStatus("");
			ImageProcessor ip;
			ip = imp.getProcessor();
			return imp;
		} catch (IOException e) {
			IJ.error("An error occured reading the file.\n \n" + e);
			IJ.showStatus("");
			return null;
		}
	}

	public static int fileInfoType(int speType) {
		switch (speType) {
			case SpeHeader.FLOAT:
				return FileInfo.GRAY32_FLOAT;
			case SpeHeader.LONG:
				return FileInfo.GRAY32_INT;
			case SpeHeader.INT:
				return FileInfo.GRAY16_SIGNED;
			case SpeHeader.UNINT:
				return FileInfo.GRAY16_UNSIGNED;
			default:
				return -1;
		}
	}

}
