import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;
import java.awt.*;

import ij.*;
import ij.process.*;
import ij.gui.*;

import ij.plugin.filter.*;
import ij.measure.*;
import ij.io.*;
import ij.plugin.frame.*;
import java.awt.geom.*;
import java.util.*;
import ij.plugin.*;

/**
 * plugin destin a trouve les centre de masse de la Roi actuel.
 *
 * @author gilles courbatieu
 *
 */
public class Center_Mass implements PlugInFilter {
	ImagePlus imp;
	RoiManager roiMan;
	PointRoi Cmass;//roi_ represantant le centre de masse
	double X,Y;//les coordonnes du centre de masse
	Analyzer analyse;//objet qui analyse la roi en fonction de diffrent paramtre voulu
	ResultsTable resTbl;//fenetre de resultat ou seront affiche les resultat de l'analyse
	FileInfo ImpInfo;//infromation sur l'image

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_ALL+ROI_REQUIRED;
	}

	public void run(ImageProcessor ip) {
		int i=0,nbC=0;
		int[] colIndex =new int[2];
		ImpInfo=imp.getOriginalFileInfo();

		//on inialise  un analyser et une resultstable ncessaire au calcule du centre
		resTbl =new ResultsTable();

		analyse =new Analyzer(imp,Analyzer.CENTER_OF_MASS,resTbl);//on mesure seulement le centre de masse
		analyse.run(ip);

		// on chercher les index de colonnes resultat
		while(i<resTbl.MAX_COLUMNS && nbC<2){
			if(resTbl.columnExists(i)) {
				colIndex[nbC]=i;
				nbC++;
			}
			i++;
		}

		// trac centre
		drawCroix(resTbl.getValueAsDouble(colIndex[0],0),resTbl.getValueAsDouble(colIndex[1],0));
//		on actualise les centre
		double Rx=ImpInfo.pixelWidth,Ry=ImpInfo.pixelHeight;//recuperation des dimension des pixel

//		calcule des coordonnes
		X=(resTbl.getValueAsDouble(colIndex[0],0))/Rx;
		Y=(resTbl.getValueAsDouble(colIndex[1],0))/Ry;

//		on affiche le centre a l'ecran au moyen d'une ROI
		Cmass=new PointRoi((int)(X+0.5),(int)(Y+0.5));
		imp.setRoi(Cmass);
		imp.updateAndRepaintWindow();

		//saveCentre(Cmass);//on sauve
		Prefs.set("user.centerx",X);
		Prefs.set("user.centery",Y);
		Prefs.savePreferences();

		colIndex=null;
		suppr();//on netoie
	}

	/************************************************************************
	 * fonction de trac du centre par une croix
	 * X , Y coordonnes du centre
	 ************************************************************************/

	public void drawCroix(double X, double Y)
	{
		//double Rx=ImpInfo.pixelWidth,Ry=ImpInfo.pixelHeight;//recuperation des dimension des pixel
		GeneralPath path = new GeneralPath();
		float x;
		float y;
		//calcul des coordonnes
		x=(float)(X);///Rx);
		y=(float)(Y);///Ry);
		// -------- affiche une croix
		//IPro.reset();//reinitialisation de l'image
		int width = imp.getWidth();
		int height = imp.getHeight();
		path.moveTo(x, 1);
		path.lineTo(x, height);
		path.moveTo(1, y);
		path.lineTo(width, y);
		ImageCanvas ic = imp.getCanvas();
		if (ic==null) return;
		if (path==null)
			ic.setDisplayList(null);
		else
			ic.setDisplayList(path, Color.red, null);
	}



	/************************************************************************
	 * procedure de suppresion de l'ensemble de attribut.
	 *
	 ************************************************************************/
	void suppr(){
		imp=null;
		Cmass=null;
		analyse=null;
		resTbl=null;
		System.gc();
	}
	/************************************************************************
	 * fonction se chargeant de la sauvegard du centre ainsi que de la Roi si il y en une
	 * @return vrai si la sauvegard a reussi faux sinon
	 ************************************************************************/
	/*synchronized boolean  saveCentre(PointRoi roi){
//		on determine le centre grace a la roi
		Point centre=new Point();
		centre.x=roi.getBoundingRect().x;
		centre.y=roi.getBoundingRect().y;

//		objet permetant l'ecriture dans un fichier
		FileWriter fileW;
		FileReader fileR;
		File Fic;
//		on recuper le nom de l'image ouverte
		FileInfo Fi=imp.getOriginalFileInfo();
		String imgName=Fi.fileName;
		
		Prefs.set("user.centerx",centre.x);
		Prefs.set("user.centery",centre.y);
		Prefs.savePreferences();
		

		final String ROI_TITLE="ROI";//titre de champ
		final String CENTER_TITLE="CENTER";//titre de champ
		final char LINE_SEPARATOR='\n';//caractre separant les ligne
		final char IMG_SEPARATOR='#';//caractre separant les le nom de l'image
		final char FIELD_SEPARATOR='?';//caractre separant les champs
		final char SUBFIELD_SERARATOR=':';//caratre separant les attribut des champs
		final String Path=IJ.getDirectory("startup")+"Save.txt";//chemins du fichier de sauvegard
		int nbChar,subFieldCounter=0;//nombre de caractre du fichier, compteur de champ lors de la lecture
		int alreadySave=-1;//a la valeur de l'index dans readBuf si l'image est deja prsant dans le fichier sinon -1

		//variable utilisaer aussi bien en lecture qu'en ecriture
		Vector Buf=new Vector();//vecteur stockant l'ensblem de ligne du fichier
		String[] Line=new String[6];//tabeau stockant l'ensemble des champ composant une ligne
		String text="";//chaine stockant les valeur des champ

		Fic=new File(Path);
//		on verifie s'il existe sinon on le cre
		if(!Fic.exists())
			{
				IJ.error("Fichier de sauvegarde absent !!");
				try
					{
						Fic.createNewFile();
					}
				catch(Exception e)
					{
						IJ.error("Problme de cration du fichier");
						return false;
					}
				//IJ.showMessage("cration russie !");
			}
		nbChar=(int)Fic.length();//on recupere le nombre de caractre deja presant dans le fichier, eventuelement 0
		//IJ.error("Fichier de sauvegarde "+Path+" "+nbChar);
		try
			{
				fileR=new FileReader(Fic);
				char c;
				for(int i=0;i<nbChar;i++)//extraction du contenu du fichier de sauvergarde
					{
						c=(char)fileR.read();
//						si on recontre un caractre de separation de champ ou de sous-champ on ajout le string au tableau
						if(c==IMG_SEPARATOR || c==SUBFIELD_SERARATOR || c==FIELD_SEPARATOR)
							{
								Line[subFieldCounter]=text;
								text="";
								subFieldCounter++;
							}
						else if(c==LINE_SEPARATOR)
							{//si on rencontre un caractre de fin de ligne on ajout le sting au tableau puis le tableau au vecteur
								Line[subFieldCounter]=text;
								text="";
								subFieldCounter=0;
								Buf.add(Line);
								Line=new String[6];
							}
							else//sinon on ajout le caractre au string
								{
									text+=c;
								}
					}
				for(int i=0;i<Buf.size();i++)
					{//on regard si l'image est deja presant
						if(imgName.equals(((String[])Buf.elementAt(i))[0]))
							{
								alreadySave=i;
							}
					}

				//on ajout a la fin ou on remplce selon le cas les info de l'image
				if(alreadySave==-1)
					{//sin non presant dans le fichier
						Line=new String[6];
						Line[0]=imgName;
						Line[1]=CENTER_TITLE;
						Line[2]=""+centre.x;
						Line[3]=""+centre.y;
						Line[4]=ROI_TITLE;
						Line[5]="null";
						Buf.add(Line);
					}
				else//si deja presant
					{
						Line=(String[])Buf.elementAt(alreadySave);
						Line[2]=""+centre.x;
						Line[3]=""+centre.y;
						Buf.setElementAt(Line,alreadySave);
					}

				//on actualise le fichier en entier
				fileW=new FileWriter(Fic);
				Fic.delete();
				Fic.createNewFile();
				for(int i=0;i<Buf.size();i++)
					{
						Line=(String[])Buf.elementAt(i);
						text=Line[0]+IMG_SEPARATOR
								+Line[1]+SUBFIELD_SERARATOR+Line[2]+SUBFIELD_SERARATOR+Line[3]+FIELD_SEPARATOR
								+Line[4]+SUBFIELD_SERARATOR+Line[5]+LINE_SEPARATOR;
						fileW.write(text.toCharArray());
						text="";
					}
				fileW.flush();
				fileW.close();
				fileR.close();
			}
		catch(Exception e)
			{
				IJ.log(e.toString());
				for(int i=0;i<(e.getStackTrace()).length;i++)
					{
						IJ.log((e.getStackTrace()[i]).toString());
					}
				return false;
			}
		return true;
	}*/
}
