import ij.IJ;
import ij.ImagePlus;
import ij.gui.GenericDialog;
import ij.measure.Calibration;
import ij.measure.Measurements;
import ij.plugin.filter.PlugInFilter;
import ij.process.Blitter;
import ij.process.ByteProcessor;
import ij.process.ImageConverter;
import ij.process.ImageProcessor;

/**
 * - Transformation of Square-Root Encoded Data to Linear Data -
 * Transformation of pixels from the PhosphorImager or FluorImager data files (.gel files)
 * to linear values.
 *
 * Formula: grayscale_value = Square(stored_value)*scale factor
 * 2 steps for each pixels:
 * 1/ multiply the pixel value by itself to square the data. You obtain a 32bit result.
 * 2/ Multiply the squared pixel value by the Scale factor. You obtain a double value.
 *
 * Note that the Scale factor is a private tag of the .gel format of files.
 * 16-bit GEL file is an extension to the standard TIFF using private tags.
 * The MD_ScalePixel tag  code is 332446
 *
 * Example of data found: 4,75624256837099E-5
 * it equals to 1/21025
 * We will use this value for Scale Factor default value
 *
 * @author Remi Cathelin SIGENAE Team - INRA - cathelin@toulouse.inra.fr
 * @version 2006/01/25
 *
 */
public class Linearize_Gel_Data implements PlugInFilter, Measurements {
	private ImagePlus imp;
	private GenericDialog gd;
	private double scaleFactor = 1.0D/21025.0D;//default value

	public int setup(String arg, ImagePlus imp) {
		if (IJ.versionLessThan("1.18o"))
			return DONE;
		this.imp = imp;
		return DOES_ALL;
	}

	public void run(ImageProcessor ip) {

		switch (imp.getBitDepth()) {

		case 16:
			//interface

			//scale factor
			scaleFactor = 1.0D/21025.0D;//default value
			//end interface
			ip = imp.getProcessor();
//			32bits conversion
			ip = ip.convertToFloat();
			ip.sqr();
			ip.multiply(scaleFactor);
			imp.setProcessor("new image",ip);
			imp.updateAndDraw();



			break;

		default:
			IJ.error("16 bits grayscale image required!");

		}


	}


}

