import ij.*;
import ij.process.*;
import ij.gui.*;

import java.awt.*;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;

import ij.io.*;
import ij.plugin.filter.*;
import ij.plugin.frame.*;

/**
 * plugin conu pour le cration et sauvegard de masque
 * @author gilles courbatieu / Olivier Tach'e  CEA Saclay 2008
 *
 */
public class Mask_Creator implements PlugInFilter {
	ImagePlus imp;
	ImageWindow IWin;//window de l'image
	GenericDialog gd;//boite de dialogue standard
	ShapeRoi GlobaleSelection,maskRoi;
	List rois;//liste de selection de roi dans l'interface RoiManager
	int nbRoi=0;//nombre de Roi reconnu par RoiManager
	boolean roiSaveOk=false,roiSaveTextOk=false;//vrai si la sauvergard a reussi faux sinon
	boolean link=false;//vrai si il n'ya pas de roiManager ouvere
	String RoiPath,imgPath,defaultName="",savePath;//different variable servant a la construction de chemin de sauvegard
	FileInfo Fi;//ncessaire pour obtenire le chmin de l'image
	RoiManager roiMan;
	Roi combineRoi=null;
	ShapeRoi RegionInterse;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_ALL;
	}

	public void run(ImageProcessor ip) {
		IWin=imp.getWindow();
		GlobaleSelection= new ShapeRoi(new Roi(0,0,imp.getWidth(),imp.getHeight()));
		//	test if ROI path exist
		String path=IJ.getDirectory("startup")+"/ROI";
		File rep=new File(path);
		if (rep.exists()!=true)
		{
			// directory not exist, create it
			IJ.showMessage("Directory "+rep.getPath()+" not exist. ImageJ created it !");
			rep.mkdir();
		}
		
		//	get path from image
		Fi= imp.getOriginalFileInfo();
		imgPath=Fi.directory+Fi.fileName;
		RoiPath="ROI\\"+Fi.fileName.replace('.', '_')+".zip";
		//	get roi manager
		roiMan =(RoiManager)WindowManager.getFrame("ROI Manager");
		if (roiMan==null) {
			IJ.error("ROI Manager is not open, no ROI selected.");
			return;
		}
		
		List roiList=roiMan.getList();
		nbRoi=roiList.getItemCount();
		if(nbRoi<=0)
		{
			IJ.error("No ROI in the ROI manager");
			return;
		}
		if(nbRoi==1){
			IJ.showMessage("ImageJ", "Actual ROI will be saved as mask.");
		}
		else
		{	// get in memory the last ROI (selection)
			roiMan.select(roiList.getItemCount()-1);
			RegionInterse=new ShapeRoi(imp.getRoi());
			roiMan.runCommand("Delete");

			//combine all masks except last one
			roiMan.runCommand("Select All");
			roiMan.runCommand("Combine");
			combineRoi=imp.getRoi();
			roiMan.runCommand("Deselect");
			roiList=roiMan.getList();
			nbRoi=roiList.getItemCount();
			// delete all masks except last one from combination
			int nb=nbRoi,i=0;
			while(nb>0)
				{
					roiMan.select(i);
					roiMan.runCommand("Delete");
					roiList=roiMan.getList();
					nb=roiList.getItemCount();
					i++;
				}
			roiList=roiMan.getList();
			if(roiList.getItemCount()>1)
				{
					IJ.error("Error during masks creation.");
					return;
				}
			//	substract masks from selection
			maskRoi=new ShapeRoi(combineRoi);
			RegionInterse=RegionInterse.not(maskRoi);
			roiMan.add(imp,RegionInterse,0);
		}
		roiMan.select(0);
		roiMan.runCommand("Rename","RoiMask");
		//	saving
		roiSaveOk=roiMan.runCommand("Save",RoiPath);
		if(!roiSaveOk)
		{
			IJ.error("Error on saving ROI.");
			return;
		}
		Prefs.set("user.ROI",RoiPath);
		Prefs.savePreferences();
		
		
	}
	/************************************************************************
	 * fonction renvoyant le nombre de roi presentes dans le roi manager
	 * @return nombre de roi dans le roi manager
	 ************************************************************************/
	int nbRoiInRoiManager(){
//		recuperation du nombre de Roi pris en charge par ImageJ
		RoiManager roiM=(RoiManager)WindowManager.getFrame("ROI Manager");
		List rois=roiM.getList();
		int nb=rois.getItemCount();
		return nb;
	}
	
	
	/************************************************************************
	 * fonction se chargeant de la sauvegard du masque si il y en un
	 * @return vrai si la sauvegard a reussi faux sinon
	 ************************************************************************/
	/*boolean save()
	{
		List roiList;
		Roi combineRoi=null;
		boolean oneRoi=false;
		ShapeRoi RegionInterse;
//		on cre les chemin d'accer
		// teste si le repsertoire existe.
		String path=IJ.getDirectory("startup")+"\\ROI";
		IJ.log(path);
		File rep=new File(path);
		if (rep.exists()!=true)
		{
			// directory not exist
			IJ.write("Directory "+rep.getPath()+" not exist. ImageJ will create it !");
			rep.mkdir();
		}
		RoiPath="ROI\\"+Fi.fileName.replace('.', '_')+".zip";
        //on retire les dernieres roi qui constitue les zones de selection
		roiList=roiMan.getList();
		nbRoi=roiList.getItemCount();
		if(nbRoi==1){
			oneRoi=IJ.showMessageWithCancel("ImageJ", "Voulez-vous sauvegarder la ROI actuelle en tant que masque ?");
			if(!oneRoi)return false;
		}
		if(!oneRoi)
			{
				roiMan.select(roiList.getItemCount()-1);
				RegionInterse=new ShapeRoi(imp.getRoi());
				roiMan.runCommand("Delete");

//				on fusionne toutes les roi ensemble
				roiMan.runCommand("Select All");
				roiMan.runCommand("Combine");
				combineRoi=imp.getRoi();

				roiMan.runCommand("Deselect");

				roiList=roiMan.getList();
				nbRoi=roiList.getItemCount();

//				on supprime toutes les roi sauf celle issu de la fusion
				int nb=nbRoi,i=0;
				while(nb>0)
					{
						roiMan.select(i);
						roiMan.runCommand("Delete");

						roiList=roiMan.getList();
						nb=roiList.getItemCount();

						i++;
					}
				roiList=roiMan.getList();
				if(roiList.getItemCount()>1)
					{
						IJ.write("Erreur dans la cration du masque !!!");
						return false;
					}
//				on soustrait celle-ci a la zone de selection precedament obtenu
				maskRoi=new ShapeRoi(combineRoi);
				RegionInterse=RegionInterse.not(maskRoi);
				roiMan.add(imp,RegionInterse,0);
			}
		roiMan.select(0);
		roiMan.runCommand("Rename","RoiMask");

//		on sauve
		roiSaveOk=roiMan.runCommand("Save",RoiPath);
		//roiSaveTextOk=saveText();
//		si probleme de sauvegard on stop
		if(!roiSaveOk)
			{
				IJ.error("Erreur lors de la sauvegarde : problme de sauvegarde des Roi !");
				return false;
			}
		if(!roiSaveTextOk)
			{
				IJ.error("Erreur lors de la sauvegarde : problme de mise a jour du fichier de sauvegarde !");
				return false;
			}

		IJ.write("chemin de sauvegarde :"+(IJ.getDirectory("startup"))+RoiPath);
		return true;
	}*/

	/************************************************************************
	 * cleaning memory
	 ************************************************************************/
	private void clean(){
		imp =null;
		IWin=null;
		gd=null;
		rois=null;
		Fi=null;
		roiMan=null;
    	System.gc();
	}
}
