import ij.*;
import ij.process.*;
import ij.gui.*;
//import ij.io.DirectoryChooser;
//import ij.io.FileInfo;
import ij.io.*;
import java.awt.*;
import java.awt.geom.GeneralPath;
import java.math.*;
import java.util.*;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;
import java.util.Iterator;

import ij.plugin.filter.*;
import ij.plugin.frame.RoiManager;
/**
 * plugin qui effectue les traitemetn de regroupement radial mais avec plusieur modalit�
 *  comme une echelle en "Q" et non en pixel, plus la possiblite de misa a l'echelle.
 * @author gilles courbatieu
 *
 */
public class Q_radial_Average implements PlugInFilter {
	ImagePlus imp;
	ImageProcessor IPro;
	GenericDialog gd;//boite de dialogue standard
	Point Centre,CentreScale;///cente de l'image
	int imgH,imgW;//respectivement hauteur et largeur de l'image
	float D,qMax;//D: distence ehcantillion/detecteur,qMax: valeur max de "q" definie par l'utilisateur
	double L,a;//L: longueur d'onde,a: dimension d'un pixel
	float qSecteur;//pas en q
	int qDiv,DisMax;//qDiv: decoupe de qMax,DisMax: distance max de coin par rappot au centre
	double[] qValue;// valeur en q des diffrent secteur
	double[] averageValue;//somme des valeurs par secteur
	float tolerance;
	double [] values; // sum of values after removing high pixels
	double [] deviation; // array of deviation
	double tempoValue;
	int[] nbPts;//nombre de point par secteur
	int removed;//nb of removed points
	int[] PixelToQ;//index en distance (pixel), contien l'index de sont equivalant en q dans qValue
	float[] qTab;
	Roi selectRoi,defaultRoi;
	FileInfo Fi;//info de l'image
	String OutputPath,RoiPath;//chemin de sortie des donn�es, et chemin du masque
	String RoiFile;//roi file name
	RoiManager roiMan;
	double prCent=0.0; //pour le progressBar
	double prCent2=0; //pour le progressBar
	double e=1,alpha=0;//coef de decalage en x et y
	boolean scaled=false;//vrai s'il y a une mise a l'echelle a faire sinon faux
	boolean qcalcul=true;
	
	

	public int setup(String arg, ImagePlus imp) {
		if(imp==null){
			IJ.noImage();
			return DONE;
		}
		this.imp = imp;
//		initialisation de variable
		imgH=imp.getHeight();
		imgW=imp.getWidth();
		Centre=new Point(0,0);
		CentreScale=new Point(0,0);
		defaultRoi=new Roi(0,0,imgW,imgH);
		return DOES_16+DOES_32+DOES_8G;
	}

	public void run(ImageProcessor ip) {
		ip.snapshot();
		IPro=ip;
		ImageProcessor ipmask;
		//on recupere les info de l'image
		Fi= imp.getOriginalFileInfo();
		//on recupere les roi manager
		roiMan =(RoiManager)WindowManager.getFrame("ROI Manager");
		if(roiMan==null) roiMan =new RoiManager();
		else{
			roiMan.close();
			roiMan = new RoiManager();
		}
				
		if (!getParameters())// on demande les parametres a l'utilisateur
			{
			CleanUp();
			return;
		}
		
		
		Rectangle maskBounds; //bounds of mask
		int xMask,yMask;//x and y coordinate of the rectangle mark
		
//				
//		
		int ind=0;
		
//		on demande du facteur de redimention pour les image gel
		int factor=-1;
		if((Fi.fileName).indexOf(".gel")!=-1)
			{
				YesNoCancelDialog YNC =new YesNoCancelDialog(new Frame(),"ImageJ","Do you want to linearize GEL image ?");
				if(YNC.yesPressed())
					{
						factor=21025;
					}
				else
					{
						if(YNC.cancelPressed()) return;
					}
			}

		//--------- regroupement radial -----------
		Date today = new Date();
		long start = today.getTime(); // beginning time
		//IJ.log(""+start);
		// test sur la roi
		ipmask=selectRoi.getMask();
		if (ipmask==null) {
	        IJ.error("Non-rectangular selection required");
	        return;
		}
		maskBounds=selectRoi.getBounds();
		xMask=maskBounds.x;
		yMask=maskBounds.y;
		
		GeneralPath path = new GeneralPath();
		IPro.reset();//reinitialisation de l'image
		int width = imp.getWidth();
		int height = imp.getHeight();
		path.moveTo(Centre.x, 1);
		path.lineTo(Centre.x, height);
		path.moveTo(1, Centre.y);
		path.lineTo(width, Centre.y);
		ImageCanvas ic = imp.getCanvas();
		if (ic==null) return;
		if (path==null)
			ic.setDisplayList(null);
		else
			ic.setDisplayList(path, Color.red, null);
		
//		 ------ on initialise le tableau
		qSecteur=qMax/(float)qDiv; // delta q
		averageValue=new double[qDiv+1];
		values=new double[qDiv+1];
		deviation=new double[qDiv+1];
		nbPts=new int[qDiv+1];
		qValue=new double[qDiv+1];

		for(int i=0;i<qDiv+1;i++){
			averageValue[i]=0;
			nbPts[i]=0;
			qValue[i]=i*qSecteur;
			values[i]=0;
			deviation[i]=0;
		}


		double[] tabXY=new double[2];
		double indice;
		double q;
		removed=0;
		int test;
		IJ.showProgress(0.0);
		IJ.showStatus("Radial averaging...");
		test=0;
		IJ.log("D:"+D+"  L:"+L);
		for(int x=0;x<=imgW;x++){
			for(int y=0;y<=imgH;y++){
				//if (ipmask.get(x,y)==0){
				//IJ.log("x:"+x+"  y:"+y+" ->"+ipmask.getf(x,y) );
				//}
				if (ipmask.getPixel(x-xMask,y-yMask)!=0)
				//if(selectRoi.contains(x,y))//on verifie si on est dans la roi
					{	
					//IJ.log("x:"+x+"  y:"+y);
						tabXY=scale(x,y);
						indice=distance(tabXY[0],tabXY[1],Centre.x,Centre.y); // calcul distance au centre
						if (qcalcul) 
							{
							q=((4*Math.PI)/L)*Math.sin(Math.atan((indice*a)/(2*D)));
							}
						else
							{
							q=indice;
							}
						if(q<=qMax)
							{//si la valeur en Q est inf ou egal au qMax
							// on trouve son indice dans le tableau
								ind=(int)(Math.floor((q/qSecteur)+0.5)); // arrondi  l'entier suprieur ou infrieur le + proche (q/qSecteur);
								//if (ind>=qDiv) IJ.log("ind:"+ind+"  q:"+q+" qDiv"+qDiv );
								averageValue[ind]+=ip.getf(x,y);
								nbPts[ind]+=1;
							}						
					}//roi
				
			}//y
			prCent=(double)(x)/(double)(imgW);
			IJ.showProgress(prCent);
						
		}//x
		
//		on effectue la moyenne des valeur des pixel en fonction de la distance
		for(int i=0;i<qDiv;i++)
		{
			averageValue[i]/=nbPts[i];
			nbPts[i]=0;
		}
		// end of first process
	
		//--------- remove hot pixels
		if (tolerance>0){
			// on essaie d'enlever les pixels chauds
			IJ.log("removing pixels");
			IJ.showProgress(0.0);
			IJ.showStatus("Radial averaging...");
			test=0;
			
			for(int x=0;x<=imgW;x++){
				for(int y=0;y<=imgH;y++){
					if (ipmask.getPixel(x-xMask,y-yMask)!=0)
					//on verifie si on est dans la roi
						{							
							tabXY=scale(x,y);
							indice=distance(tabXY[0],tabXY[1],Centre.x,Centre.y); // calcul distance au centre
							if (qcalcul) 
								{
								q=((4*Math.PI)/L)*Math.sin(Math.atan((indice*a)/(2*D)));
								}
							else{
								q=indice;
							}
							if(q<=qMax)
								{//si la valeur en Q est inf ou egal au qMax
								// on trouve son indice dans le tableau
									ind=(int)(Math.floor((q/qSecteur)+0.5)); // arrondi  l'entier suprieur ou infrieur le + proche (q/qSecteur);
									tempoValue=ip.getf(x,y);
									if((tempoValue>=averageValue[ind]*(1-tolerance/100)) & (tempoValue<=averageValue[ind]*(1+tolerance/100)))
									{
											values[ind]+=tempoValue;
											nbPts[ind]+=1;
									}
									else
									{
										removed+=1;
										path.moveTo(x, 1);
										path.moveTo(x, height);
										path.moveTo(1, y);
										path.moveTo(width, y);
										/*ic = imp.getCanvas();
										if (ic==null) return;
										if (path==null)
											ic.setDisplayList(null);
										else*/
										ic.setDisplayList(path, Color.yellow, null);
										//IJ.log("pixel x="+tabXY[0]+"   y="+tabXY[1]+", value="+tempoValue+", average="+averageValue[ind]+"  skipped");
									}
								}
									
							}//roi
							
					}// fin y
					prCent=(double)(x)/(double)(imgW);
					IJ.showProgress(prCent);
				
				} //fin x
			
			IJ.log("nb of pixel removed :"+removed);
			//on effectue la moyenne des valeurs des pixel en fonction de la distance
			for(int i=0;i<qDiv;i++){
				values[i]=values[i]/nbPts[i]; // copy of values array
				nbPts[i]=0;
			}
		}// end of test tolerance
		
		//--------- standard deviation
		IJ.showProgress(0.0);
		IJ.showStatus("Standard deviation...");
		for(int x=0;x<=imgW;x++){
			for(int y=0;y<=imgH;y++){
				if (ipmask.getPixel(x-xMask,y-yMask)!=0)
				//on verifie si on est dans la roi
					{	
						tabXY=scale(x,y);
						indice=distance(tabXY[0],tabXY[1],Centre.x,Centre.y); // calcul distance au centre
						if (qcalcul) 
							{
							q=((4*Math.PI)/L)*Math.sin(Math.atan((indice*a)/(2*D)));
							}
						else
							{
							q=indice;
							}
						if(q<=qMax)
							{//si la valeur en Q est inf ou egal au qMax
							// on trouve son indice dans le tableau
								ind=(int)(Math.floor((q/qSecteur)+0.5)); // arrondi  l'entier suprieur ou infrieur le + proche (q/qSecteur);
								tempoValue=ip.getf(x,y);
								if (tolerance>0)
								{
									if((tempoValue>=averageValue[ind]*(1-tolerance/100)) & (tempoValue<=averageValue[ind]*(1+tolerance/100)))
									{
											//values[ind]+=tempoValue;
											deviation[ind]+=Math.pow(ip.getf(x,y)-values[ind],2);
											nbPts[ind]+=1;
									}
								}
								else
								{
									deviation[ind]+=Math.pow(ip.getf(x,y)-averageValue[ind],2);
									nbPts[ind]+=1;
								}
							}
					}// boucle ROI
						
				}// boucle y
			
				prCent=(double)(x)/(double)(imgW);
				IJ.showProgress(prCent);
			
			
		}// boucle x
		//	on effectue le calcul de l'erreur en fonction de la distance
		for(int i=0;i<qDiv;i++){
			deviation[i]=Math.pow(deviation[i],0.5);
			deviation[i]/=nbPts[i];
			if (tolerance>0)averageValue[i]=values[i];
			
		}
		
		
		
		IJ.showProgress(1.1);
		IJ.showStatus("");
		IJ.showTime(imp,start,"Elapsed time : ");
		//DrawGraphics("Q Radial Average","Q("+IJ.angstromSymbol+")","I",qValue,averageValue,1,Color.BLUE,ip);
		WriteResulte();
        CleanUp();//on libere la memoire
        

	}



	/************************************************************************
	 * procedure qui calcule les coodonn�es r�el d'un point
	 * en fonction du paramtre de decalage
	 * @param x coordonn�es du point a mettre a l'echelle (en pixel)
	 * @param y
	 *************************************************************************/
	double[] scale(final int X,final int Y){
		double[] scale= new double[2];
		double x=X;
		double y=Y;
		if(!scaled||e==1){//si il n'y a pas l'echelle
			x=X;
			y=Y;
		}
		else
		{
			// operation sur Y
			y=(Y-Centre.y)*e+Centre.y;
			//if (y>0){
			//		}
			//D*Math.cos(alpha)
			//
		}

		/**
		double x1,y1,x2,y2,distance_centre,angle_centre;

//		on affecte le point d'une rotation d'angle equivalant a l'inclin�sion de l'ellipse
		x1=(int)(x*Math.cos(-alpha)-y*Math.sin(-alpha));
		y1=(int)(y*Math.cos(-alpha)+x*Math.sin(-alpha));

//		calcule de carre
		x2=x1*x1;
		y2=y1*y1;

		distance_centre=Math.sqrt(x2+y2);//distance au centre
		angle_centre=Math.atan((float)y/x);//angle au centre

//		on calcule les coordonn�es r�el (transformation de l'ellipse en cercle)
		x1=distance_centre*Math.cos(angle_centre)+Centre.x;
		y1=e*distance_centre*Math.sin(angle_centre)+Centre.y;

//		et on retrourne les valeur
		 * */

		scale[0]=x;
		scale[1]=y;
		return scale;
	}
	/************************************************************************
	 * fonction qui calcule la distance entre 2 point en l'arrondissant a l'entier inferieur
	 * @param x coordnn�es du 1er point
	 * @param y coordnn�es du 1er point
	 * @param x2 coordnn�es du 2eme point
	 * @param y2 coordnn�es du 2eme point
	 * @return entier representant la distance entre les 2 point
	 *************************************************************************/
	double distance(double x,double y,double x2,double y2){
		double X=0,Y=0,res=0;
		Y=Math.pow((y-y2),2);
		X=Math.pow((x-x2),2);
		res=Math.sqrt(X+Y)+0.5;
		return res;
	}
	/************************************************************************
	 * fonction que s'occupe de la convertion Q/pixel ou pixel/Q en fonction du type
	 * @param PixQ valeur que l'on veut convertir en pixel ou Q
	 * @param type indique la dimension d'entr�
	 * @return
	 ************************************************************************/
	double pixelQ(int PixQ,char type){
		if(type=='p'||type=='P')return (((4*Math.PI)/L)*((PixQ*a)/(2*D)));
		else return ((2*D*PixQ*L)/(4*Math.PI*a));
	}
	/************************************************************************
	 * fonction qui s'occupe de la convertion Q/pixel
	 * @param q valeur que l'on veut convertir en pixel
	 * @return
	 ************************************************************************/
	int Q2pixel(double q)
	{
		return (int)((2*D*q*L)/(4*Math.PI*a));
	}

	
	/************************************************************************
	 * Parameters GUI
	 * @return true if ok, False if canceled
	 ************************************************************************/
	boolean getParameters(){
		OpenDialog ODRoiFile;
		DirectoryChooser ODOutputPath;
	
		//extract pixel size
		a=Fi.pixelHeight;
		if(a<0){
			IJ.error("Error, pixel size is incorrect !!");
			return false;
		}

		Double convertor= new Double(0);//convertisseur de flotant sous forme de <String> en entier sous forme de <double>
		MathContext mc= new MathContext(2);//precision
		BigDecimal virgule=new BigDecimal(0);//premet de gere le nombre de decimal d'un flotant
		gd = new GenericDialog("Parameters...");
		
		String[] TabPathRoi={Prefs.get("user.ROI","null"),"other"};
		String[] TabPathOutput={Prefs.get("user.Output.path","null"),"other"};
		a=Prefs.get("user.PixelSize",1.0);
		L=Prefs.get("user.wavelength",1.542);
		D=(float)Prefs.get("user.DetectorDistance",1.0);
		qDiv=(int)Prefs.get("user.QDiv",512);
		e=(float)Prefs.get("user.e",1);
		Centre.x=(int)Prefs.get("user.centerx",1);
		Centre.y=(int)Prefs.get("user.centery",1);
		qMax=(float)Prefs.get("user.qmax",1.0);
		qcalcul=(boolean)Prefs.get("user.typeofcalcul",true);
		String[] typeChoice={"q","pixel"};
		tolerance=-1;
		//IJ.log("distancemax:"+DisMax);
		//double tmpQ=(((4*Math.PI)/L)*((DisMax*a)/(2*D)));
		// DIALOG construction --------------------------------
		gd.addChoice("ROI :",TabPathRoi,TabPathRoi[0]);
		gd.addChoice("Output path :",TabPathOutput,TabPathOutput[0]);
		gd.addNumericField("Center x : ",Centre.x,0);
		gd.addNumericField("Center y : ",Centre.y,0);
		gd.addNumericField("scaling x/y : ",e,4);
		gd.addNumericField("Wavelength ("+IJ.angstromSymbol+") : ",L,4);
		gd.addNumericField("Detector distance (cm) : ",D,4);
		gd.addNumericField("Pixel size (cm) : ",a,4);
		//gd.addMessage("calculated qmax="+tmpQ);
		gd.addNumericField("Qmax or pixel max : ",qMax,4);
		gd.addNumericField("Number of points : ",qDiv,4);
		
		int choice=0;
		if (!qcalcul) choice=1;
		gd.addChoice("Type of averaging :",typeChoice,typeChoice[choice]);
		gd.addNumericField("Tolerance for hot pixels (%)",tolerance,4);
		gd.showDialog();
		
		// End of dialog --------------------------------
		if(gd.wasCanceled()){
			return false;
		}
		//	get parameters
		RoiFile=gd.getNextChoice();
		if (RoiFile=="other")
		{
			//ask for ROI
			ODRoiFile=new OpenDialog("Select ROI file","");
			RoiFile=ODRoiFile.getDirectory()+ODRoiFile.getFileName();
			if (RoiFile==null) {
				IJ.error("No ROI File selected.");
				return false;	
			}
			
		}
		Prefs.set("user.ROI",RoiFile);
		roiMan.runCommand("Open",RoiFile);
		roiMan.select(0);
		selectRoi=imp.getRoi();
		if (selectRoi==null) IJ.error("error on loading ROI");
		
		OutputPath=gd.getNextChoice();
		if (OutputPath=="other")
		{
			//ask for output path
			ODOutputPath=new DirectoryChooser("Select output path");
			OutputPath=ODOutputPath.getDirectory();
			if (OutputPath==null)
				{
				IJ.error("No output path selected.");
				return false;	
				}
			
			
		}
		Prefs.set("user.Output.path",OutputPath);
		
		Centre.x=(int)gd.getNextNumber();
		Centre.y=(int)gd.getNextNumber();
		e=gd.getNextNumber();
		L=gd.getNextNumber();
		if(L<=0)return getParameters();

		D=(float)gd.getNextNumber();
		if(D<=0)return getParameters();

		a=(float)gd.getNextNumber();
		if(a<=0)return getParameters();

		qMax=(float)gd.getNextNumber();
		if(D<=0)return getParameters();

		qDiv=(int)gd.getNextNumber();
		if(qDiv<=0)return getParameters();
		
		String typeOfCalcul=gd.getNextChoice();
		qcalcul=(typeOfCalcul=="q");
		tolerance=(float)gd.getNextNumber();

		//on sauve les parametres entr�s
		Prefs.set("user.wavelength",L);
		Prefs.set("user.DetectorDistance",D);
		Prefs.set("user.Qmax",qMax);
		Prefs.set("user.QDiv",qDiv);
		Prefs.set("user.PixelSize",a);
		Prefs.set("user.e",e);
		Prefs.set("user.centerx",Centre.x);
		Prefs.set("user.centery",Centre.y);
		Prefs.set("user.qmax",qMax);
		Prefs.set("user.typeofcalcul",qcalcul);
		Prefs.savePreferences();


		return true;
	}
	/************************************************************************
	 * fonction qui ecrit les resultat du traitement dans un fichier
	 * @return vrai si reussit faux sinon
	 *************************************************************************/
	synchronized boolean  WriteResulte(){
		final int NBLINE=averageValue.length-1;//constant de nombre de ligne maximum dans les fichier
		final String LINE_SEPARATOR=System.getProperty("line.separator");//recuperation du separteur de ligne en fonction du system
		String SPACE="\t",FIELD1="Q",FIELD2="Intensity",FIELD3="Nb of points",FIELD4="AbsDeviation",FILETYPE=".rgr";//constant servant a l'ericture dans le fichier
		if (!qcalcul) FIELD1="Pixel"; 
		int NbIgnored=0;//nombre de ligne non ecrite dans le fichier
		File Fic;
		FileWriter fileW;
		String Path,Line,valueGr="",valueDs="",valuePt="",valueDeviation="";
		DirectoryChooser dir;
		
//		on recuper le chemin de l'image contenu dans le fichier de preference
		String loadErr=Prefs.load(this,IJ.getApplet());
		if(loadErr!=null){
			IJ.error("Error during reading preferences :"+loadErr);
			return false;
		}

		
		Path=OutputPath;
		Path+=Fi.fileName.replace('.', '_')+FILETYPE;
		//	}
//		on demand si on veu actualiser le fichier de sorti
		Fic=new File(Path);
		if(Fic.exists()){
			GenericDialog gd = new GenericDialog("ImageJ");
			gd.addMessage("File exist, replace it ?");
			gd.showDialog();
			if(!gd.wasCanceled()){
				Fic.delete();
			}
			else return false;
		}
//		ecriture du fichier
		try{
			Fic.createNewFile();
			fileW=new FileWriter(Fic);
			Line="#"+FIELD1+SPACE+FIELD2+SPACE+FIELD3+SPACE+FIELD4+LINE_SEPARATOR;
			fileW.write(Line);
			for(int i=0;i<NBLINE;i++){
				valueGr=valueGr.valueOf(averageValue[i]);
				if(!valueGr.equals("NaN")){
				valueDs=valueDs.valueOf(qValue[i]);
				valuePt=valuePt.valueOf(nbPts[i]);
				valueDeviation=valueDeviation.valueOf(deviation[i]);
				Line=valueDs+SPACE+valueGr+SPACE+valuePt+SPACE+valueDeviation+LINE_SEPARATOR;
				fileW.write(Line);
				}
				else NbIgnored++;
			}
			fileW.flush();
			fileW.close();
		}
		catch(Exception e){
			IJ.log(e.toString());
			for(int i=0;i<(e.getStackTrace()).length;i++){
				IJ.log((e.getStackTrace()[i]).toString());
			}
			return false;
		}
		return true;
	}
	/**
	 * procedure permettant de dessin un graphique avec les donn�es issues du traitement
	 * @param title titre du graph
	 * @param xLbl etiquette de l'axe des x
	 * @param yLbl etiquette de l'axe des y
	 * @param x valeurs en x
	 * @param y valeurs en y
	 * @param lineW epaisseur de la ligne de trace
	 * @param clr couleur de cette m�me ligne
	 */
	void DrawGraphics(String title,String xLbl,String yLbl,double[] x,double[] y,int lineW,Color clr,ImageProcessor ip){
		YesNoCancelDialog YNC = new YesNoCancelDialog(imp.getWindow(),"ImageJ","Do you want to plot datas ?");
		if(YNC.yesPressed())
			{
//				on recherche les limite de l'axe y
				double yMin=ip.getMax(),yMax=0;
				for(int i=0;i<y.length;i++){
					if(yMin>y[i]) yMin=y[i];
					if(yMax<y[i]) yMax=y[i];
				}

//				on determine le nombre de valeur de y etant indefinie i.e ="NaN"
				int realsize=0;
				Double number=new Double(0);
				for(int i=0;i<y.length;i++){
					number=number.valueOf(y[i]);
					if(number.isNaN())realsize++;
				}
				realsize=y.length-realsize;

//				on reforme les tableau
				double[] xTrunc=new double[realsize];
				double[] yTrunc=new double[realsize];
				int ind=0;
				for(int i=0;i<y.length;i++){
					number=number.valueOf(y[i]);
					if(!number.isNaN()){
						xTrunc[ind]=x[i];
						yTrunc[ind]=y[i];
						ind++;
					}
				}

//				on affiche le graphe
				PlotWindow pWin= new PlotWindow(title,xLbl,yLbl,xTrunc,yTrunc);
				pWin.setLimits(0,qDiv*qSecteur,yMin,yMax);
				pWin.setLineWidth(lineW);
				pWin.setColor(clr);
				pWin.draw();
			}
	}
	/************************************************************************
	 * procedure supprimant les attribu de l'objet pour libr�re la memoire
	 ************************************************************************/
	void CleanUp(){
		imp=null;
		gd=null;
		Centre=null;
		qValue=null;
		averageValue=null;
		nbPts=null;
		PixelToQ=null;
		selectRoi=null;
		defaultRoi=null;
		Fi=null;
		OutputPath=null;
		roiMan=null;
    	System.gc();
	}
}
