import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.io.*;

import java.awt.*;
import java.io.File;
import java.io.FileReader;
import java.lang.Math;
import java.math.BigDecimal;
import java.math.MathContext;
import java.util.Vector;
import java.awt.geom.*;
import java.util.*;

import ij.plugin.filter.*;
import ij.plugin.frame.RoiManager;
/**
 * draw ellipses on image
 * @authors Olivier TACHE & Olivier SPALLA / CEA / LIONS 13-05-2011
 *
 */
public class Scale_Ellipse implements PlugInFilter , DialogListener{
	ImagePlus imp;
	ImageProcessor IPro;
	double pi=Math.PI;
	double alpha_deg=90.0;// alpha in deg
	double alpha_rad=alpha_deg*2*pi/360.0;// alpha in radians
	double theta=0.0;//scattering angle
	GenericDialog gd,gd2;//standard dialog box
	int xC,yC; // beam center position
	Point Centre=new Point();
	double ay=0,ax=0,q=0;
	double L=0; // sample to detector distance in cm
	double pixelSize=0; // pixel size in cm
	double qfirstorder=0.0;//
	double qnextorder=0.0;
	double lambda=0.0;//
	double radius_height=0,radius_factor=0.0;
	String loading=new String();
	OvalRoi oval;
	Line lineax,lineay;
	int noe=1;


	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_ALL;
	}

	public void run(ImageProcessor ip) {
		ip.snapshot();
		pixelSize=(imp.getOriginalFileInfo()).pixelHeight;
		IPro=ip;
		loadPrefs();
		Draw_Ellipse();
		if(GUIscale()){
			save();
		}
		IPro.reset();
		IJ.freeMemory();
	}
	/************************************************************************
	 * Show the dialog box
	 * 
	 ************************************************************************/
	boolean GUIscale()
	{
		IJ.freeMemory();
		double d1=0,d2=0,theta=0,tmp1,tmp2;
		gd = new GenericDialog("Ellipse parameters...");
		gd.addMessage("Center :");
		gd.addNumericField("x :",xC,0);
		gd.addNumericField("y :",yC,0);
		gd.addMessage("SAXS parameters :");
		gd.addNumericField("alpha (deg):",alpha_deg,5);
		gd.addNumericField("q for first order to be draw ("+IJ.angstromSymbol+"-1)",qfirstorder,5);
		gd.addNumericField("Sample to detector distance (cm) :",L,5);
		gd.addNumericField("pixel_size (cm) :",pixelSize,5);
		gd.addNumericField("Wavelength ("+IJ.angstromSymbol+") :",lambda,5);
		gd.addNumericField("nb of ellipses to draw : ",noe,0);
		
		gd.addDialogListener(this);
		gd.showDialog();
		if(!gd.wasCanceled())
			{
				// when user click on cancel
				return UserValueChanged(gd,true);
			}
		else {
			// delete ellipse
			ImageCanvas ic = imp.getCanvas();
			if (ic!=null) ic.setDisplayList(null);
		}
		
		return false;
	}
	/************************************************************************
	 * User Value changed
	 * gd GenericDialog
	 * askAgain true : ask again for valid values (user click on ok)
	 *************************************************************************/
	public boolean UserValueChanged(GenericDialog gd,boolean askAgain){
		// when user value changed
		xC=(int)gd.getNextNumber();
		yC=(int)gd.getNextNumber();
		alpha_deg=(double)gd.getNextNumber();
		qfirstorder=(double)gd.getNextNumber();
		L=(double)gd.getNextNumber();
		pixelSize=(double)gd.getNextNumber();
		lambda=(double)gd.getNextNumber();
		noe=(int)gd.getNextNumber();
		if(xC<0||xC>imp.getWidth()||yC<0||yC>imp.getHeight()){
			
			if (askAgain){
				IJ.error("Invalid values.");
				IJ.freeMemory();
				return GUIscale();
			}
			else 
				return true;
		}
		radius_height=ax;
		Draw_Ellipse();//xC,yC,(int)radius_height,(double)radius_factor,(double)eccentricity_factor,noe);
		
		return true;	
		
	}
	
	/************************************************************************
	 * dialog Item changed
	 *************************************************************************/
	public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
		
		return UserValueChanged(gd,false);
	}
	
	
	/************************************************************************
	 * draw ellipses
	 * @axram x ellipse center
	 * @axram y
	 *  
	 * noe number of ellipses
	 ************************************************************************/
	void Draw_Ellipse(){
		double ellipse_height,ellipse_width;
		double newx,newy; // new center coordinate
		GeneralPath path = new GeneralPath();
		IPro.reset();//image init (erase previous drawings)
		
		// ---  draw cross for center
		int width = imp.getWidth();
		int height = imp.getHeight();
		path.moveTo(xC, 1);
		path.lineTo(xC, height);
		path.moveTo(1, yC);
		path.lineTo(width, yC);
		
		// --- draw ellipses
		alpha_rad=alpha_deg*2*pi/360.0;
		for(int i=1;i<=noe;i++){
			qnextorder=qfirstorder*i;
			theta=Math.asin((qnextorder*lambda)/(4*pi))*2;
			// ------  calculate width and height
			ellipse_height=2*L*Math.tan(theta)/pixelSize;
			ellipse_width=(L*2*Math.sin(theta)*Math.cos(theta)*Math.sin(alpha_rad))/(Math.pow(Math.sin(alpha_rad), 2)-Math.pow(Math.sin(theta), 2));
			ellipse_width=ellipse_width/pixelSize;
			//IJ.log("n:"+n);
			// ------  calculate new center
			newx=xC+((L/pixelSize)*Math.pow(Math.sin(theta), 2)*Math.cos(alpha_rad))/(Math.pow(Math.sin(alpha_rad), 2)-Math.pow(Math.sin(theta), 2));
			//IJ.log("i :"+i+" ellipse height : "+ellipse_height +"ellipse width : "+ellipse_width +" new center x : "+newx);
			path.moveTo(newx, yC-50);
			path.lineTo(newx, yC+50);
			
			//x_upper_left_corner=x-radius_right*i;
			//y_upper_left_corner=y-radius_height*i;
			//arc_height=radius_height*2;
			//ellipse_height=radius_height*2*i;
			//ellipse_width=ellipse_height*radius_factor;
			path.append(new Ellipse2D.Float((float)(newx-ellipse_width/2), (float)(yC-ellipse_height/2), (float)ellipse_width,(float)ellipse_height), false);
		    
			ImageCanvas ic = imp.getCanvas();
			if (ic==null) return;
			if (path==null)
				ic.setDisplayList(null);
			else
				ic.setDisplayList(path, Color.red, null);
		
		}
	}
	/************************************************************************
	 * save parameters in user preferences
	 * 
	 ************************************************************************/
	boolean save(){
		String Err= Prefs.load(this, IJ.getApplet());
		if(Err!=null){
			IJ.error("cannot save datas !");
			return false;
		}
		
		Prefs.set("user.centerx",xC);
		Prefs.set("user.centery",yC);
		Prefs.set("user.alpha_deg", alpha_deg);
		Prefs.set("user.qfirstorder", qfirstorder);
		Prefs.set("user.DetectorDistance", L);
		Prefs.set("user.PixelSize", pixelSize);
		Prefs.set("user.wavelength", lambda);
		Prefs.set("user.noe", noe);
		
		Prefs.savePreferences();
		return true;
	}
	
	/************************************************************************
	 * get parameters
	 ************************************************************************/
	synchronized boolean loadPrefs(){
		lambda=Prefs.get("user.wavelength",1.542);
		L=(double)Prefs.get("user.DetectorDistance",1.0);
		xC=(int)Prefs.get("user.centerx",1);
		yC=(int)Prefs.get("user.centery",1);
		alpha_deg=(double)Prefs.get("user.alpha_deg",90.0);
		qfirstorder=(double)Prefs.get("user.qfirstorder",0.1525 );
		pixelSize=(double)Prefs.get("user.PixelSize", 1.0);
		noe=(int)Prefs.get("user.noe", 1);
		
		return true;
		
	}
	
	
	
	
	

}
